#include <math.h>
#include <sys/time.h>
#include <time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "pdm.h"
#include "pdm_priv.h"
#include "pdm_mpi.h"
#include "pdm_config.h"
#include "pdm_part.h"

#include "pdm_writer.h"
#include "pdm_part_to_block.h"
#include "pdm_poly_surf_gen.h"
#include "pdm_overlay.h"
#include "pdm_printf.h"
#include "pdm_error.h"
#include "pdm_mpi_node_first_rank.h"
#include "pdm_priv.h"
#include "pdm_points_merge.h"
#include "pdm_gnum.h"
#include "pdm_poly_surf_gen.h"
#include "pdm_geom_elem.h"

/*============================================================================
 * Type definitions
 *============================================================================*/

/*============================================================================
 * Private function definitions
 *============================================================================*/
static double random01(void)
{
  int sign;
  int rsigna = rand();
  int rsignb = rand();
  sign = (rsigna - rsignb) / PDM_ABS(rsigna - rsignb);
  double resultat = sign*((double)rand())/((double)RAND_MAX);
  return resultat;
}

static void
_export_ol_mesh
(
 const PDM_MPI_Comm pdm_mpi_comm,
 const int pdm_id,
 int           nFace[2][1],
 double  **sFieldOlA,
 double  **rFieldOlB,
 const int      n_part
 )
{

  int i_rank;
  int numProcs;

  PDM_MPI_Comm_rank (PDM_MPI_COMM_WORLD, &i_rank);
  PDM_MPI_Comm_size (PDM_MPI_COMM_WORLD, &numProcs);

  /*
   *  Export Mesh to Ensight
   */

  int id_cs[2];

  id_cs[0] = PDM_writer_create ("Ensight",
                                PDM_WRITER_FMT_ASCII,
                                PDM_WRITER_TOPO_CONSTANTE,
                                PDM_WRITER_OFF,
                                "test_2d_unit_ens",
                                "olmesh1",
                                pdm_mpi_comm,
                                PDM_IO_ACCES_MPI_SIMPLE,
                                1.,
                                NULL);

  id_cs[1] = PDM_writer_create ("Ensight",
                                PDM_WRITER_FMT_ASCII,
                                PDM_WRITER_TOPO_CONSTANTE,
                                PDM_WRITER_OFF,
                                "test_2d_unit_ens",
                                "olmesh2",
                                pdm_mpi_comm,
                                PDM_IO_ACCES_MPI_SIMPLE,
                                1,
                                NULL);

  /*
   * Creation des variables
   */

  int id_var_num_part[2];
  int id_var_match[2];
  int id_var_cell_match[2];
  int id_var_origin[2];
  int id_var_field[2];
  int id_geom[2];

  for (int imesh = 0; imesh < 2; imesh++) {

    if (imesh == 0) {
      id_var_field[imesh] = PDM_writer_var_create (id_cs[imesh],
                                                   PDM_WRITER_OFF,
                                                   PDM_WRITER_VAR_SCALAIRE,
                                                   PDM_WRITER_VAR_ELEMENTS,
                                                   "sOlField");
    }
    else {

      id_var_field[imesh] = PDM_writer_var_create (id_cs[imesh],
                                                   PDM_WRITER_OFF,
                                                   PDM_WRITER_VAR_SCALAIRE,
                                                   PDM_WRITER_VAR_ELEMENTS,
                                                   "rOlField");
    }

    id_var_num_part[imesh] = PDM_writer_var_create (id_cs[imesh],
                                                    PDM_WRITER_OFF,
                                                    PDM_WRITER_VAR_SCALAIRE,
                                                    PDM_WRITER_VAR_ELEMENTS,
                                                    "num_part");

    id_var_match[imesh] = PDM_writer_var_create (id_cs[imesh],
                                                 PDM_WRITER_OFF,
                                                 PDM_WRITER_VAR_SCALAIRE,
                                                 PDM_WRITER_VAR_ELEMENTS,
                                                 "matching");

    id_var_cell_match[imesh] = PDM_writer_var_create (id_cs[imesh],
                                                      PDM_WRITER_OFF,
                                                      PDM_WRITER_VAR_SCALAIRE,
                                                      PDM_WRITER_VAR_ELEMENTS,
                                                      "cell_matching");

    id_var_origin[imesh] = PDM_writer_var_create (id_cs[imesh],
                                                  PDM_WRITER_OFF,
                                                  PDM_WRITER_VAR_SCALAIRE,
                                                  PDM_WRITER_VAR_ELEMENTS,
                                                  "origin");

    /*
     * Creation de la geometrie
     */

    char nom_geom[8];
    PDM_ol_mesh_t mesht;

    if (imesh == 0) {
      strcpy (nom_geom,"olmesh1");
      mesht = PDM_OL_MESH_A;
    }

    else {
      strcpy (nom_geom,"olmesh2");
      mesht = PDM_OL_MESH_B;
    }

    id_geom[imesh] = PDM_writer_geom_create (id_cs[imesh],
                                             nom_geom,
                                             PDM_WRITER_OFF,
                                             PDM_WRITER_OFF,
                                             n_part);
    int *n_part_procs = (int *) malloc(sizeof(int) * numProcs);

    PDM_MPI_Allgather ((void *) &n_part,      1, PDM_MPI_INT,
                       (void *) n_part_procs, 1, PDM_MPI_INT,
                       PDM_MPI_COMM_WORLD);

    int *debPartProcs = (int *) malloc(sizeof(int) * (numProcs + 1));

    debPartProcs[0] = 0;
    for (int i = 0; i < numProcs; i++) {
      debPartProcs[i+1] = debPartProcs[i] + n_part_procs[i];
    }

    free(n_part_procs);

    /*
     * Debut des ecritures
     */

    PDM_g_num_t    nGOlFace;
    PDM_g_num_t    nGOlVtx;

    PDM_ol_mesh_dim_get (pdm_id,
                         mesht,
                         &nGOlFace,
                         &nGOlVtx);

    int **_olface_nb =  malloc(sizeof(int *) * n_part);
    int **_olface_idx =  malloc(sizeof(int *) * n_part);
    PDM_real_t **val_num_part = (PDM_real_t **) malloc (sizeof(PDM_real_t *) * n_part);
    PDM_real_t **val_match = (PDM_real_t **) malloc (sizeof(PDM_real_t *) * n_part);
    PDM_real_t **val_cell_match = (PDM_real_t **) malloc (sizeof(PDM_real_t *) * n_part);
    PDM_real_t **val_origin = (PDM_real_t **) malloc (sizeof(PDM_real_t *) * n_part);
    PDM_writer_step_beg (id_cs[imesh], 0.);

    for (int ipart = 0; ipart < n_part; ipart++) {

      int           nOlFace;
      int           nOlLinkedFace;
      int           nOlVtx;
      int           sOlFaceIniVtx;
      int           sOlface_vtx;
      int           sInitToOlFace;

      PDM_ol_part_mesh_dim_get (pdm_id,
                                mesht,
                                ipart,
                                &nOlFace,
                                &nOlLinkedFace,
                                &nOlVtx,
                                &sOlFaceIniVtx,
                                &sOlface_vtx,
                                &sInitToOlFace);

      int            *olFaceIniVtxIdx;
      int            *olFaceIniVtx;
      int            *olface_vtx_idx;
      int            *olface_vtx;
      int            *olLinkedface_procIdx;
      int            *olLinkedFace;
      PDM_g_num_t     *olface_ln_to_gn;
      double         *olCoords;
      PDM_g_num_t     *olvtx_ln_to_gn;
      int            *initToOlFaceIdx;
      int            *initToOlFace;

      PDM_ol_mesh_entities_get (pdm_id,
                                mesht,
                                ipart,
                                &olFaceIniVtxIdx,
                                &olFaceIniVtx,
                                &olface_vtx_idx,
                                &olface_vtx,
                                &olLinkedface_procIdx,
                                &olLinkedFace,
                                &olface_ln_to_gn,
                                &olCoords,
                                &olvtx_ln_to_gn,
                                &initToOlFaceIdx,
                                &initToOlFace);


      val_num_part[ipart] = (PDM_real_t *) malloc (sizeof(PDM_real_t) * nOlFace);
      val_match[ipart] = (PDM_real_t *) malloc (sizeof(PDM_real_t) * nOlFace);
      val_cell_match[ipart] = (PDM_real_t *) malloc (sizeof(PDM_real_t) * nOlFace);
      val_origin[ipart] = (PDM_real_t *) malloc (sizeof(PDM_real_t) * nOlFace);

      PDM_writer_geom_coord_set (id_cs[imesh],
                                 id_geom[imesh],
                                 ipart,
                                 nOlVtx,
                                 olCoords,
                                 olvtx_ln_to_gn);

      _olface_nb[ipart] = malloc(sizeof(int) * nOlFace);
      _olface_idx[ipart] = malloc(sizeof(int) * nOlFace);

      for (int j = 0; j < nOlFace; j++) {
        _olface_nb[ipart][j]  = olface_vtx_idx[j+1] - olface_vtx_idx[j];
        _olface_idx[ipart][j] = olface_vtx_idx[j] + 1;
      }

      PDM_writer_geom_faces_facesom_add (id_cs[imesh],
                                         id_geom[imesh],
                                         ipart,
                                         nOlFace,
                                         _olface_idx[ipart],
                                         _olface_nb[ipart],
                                         olface_vtx,
                                         olface_ln_to_gn);

      for (int i = 0; i < nOlFace; i++) {
        val_num_part[ipart][i] = ipart + 1 + debPartProcs[i_rank];
        val_origin[ipart][i] = -1;
        val_match[ipart][i] = -1;
        val_cell_match[ipart][i] = -1;
      }

      for (int i = 0; i < nOlLinkedFace; i++) {
        val_match[ipart][olLinkedFace[4*i]-1] = 100;
        val_cell_match[ipart][olLinkedFace[4*i]-1] = olLinkedFace[4*i + 2];
      }

      for (int i = 0; i < nFace[imesh][ipart]; i++) {
        for (int j = initToOlFaceIdx[i]; j < initToOlFaceIdx[i+1]; j++) {
          val_origin[ipart][initToOlFace[j]-1] = i;
        }
      }

      if (imesh == 0) {
        PDM_writer_var_set (id_cs[imesh],
                            id_var_field[imesh],
                            id_geom[imesh],
                            ipart,
                            sFieldOlA[ipart]);
      }
      else {
        PDM_writer_var_set (id_cs[imesh],
                            id_var_field[imesh],
                            id_geom[imesh],
                            ipart,
                            rFieldOlB[ipart]);
      }

      PDM_writer_var_set (id_cs[imesh],
                          id_var_num_part[imesh],
                          id_geom[imesh],
                          ipart,
                          val_num_part[ipart]);

      PDM_writer_var_set (id_cs[imesh],
                          id_var_match[imesh],
                          id_geom[imesh],
                          ipart,
                          val_match[ipart]);

      PDM_writer_var_set (id_cs[imesh],
                          id_var_cell_match[imesh],
                          id_geom[imesh],
                          ipart,
                          val_cell_match[ipart]);

      PDM_writer_var_set (id_cs[imesh],
                          id_var_origin[imesh],
                          id_geom[imesh],
                          ipart,
                          val_origin[ipart]);


    }

    PDM_writer_geom_write(id_cs[imesh],
                          id_geom[imesh]);

    for (int ipart = 0; ipart < n_part; ipart++) {
      free (_olface_nb[ipart]);
      free (_olface_idx[ipart]);
    }

    PDM_writer_var_write (id_cs[imesh],
                          id_var_num_part[imesh]);

    PDM_writer_var_write (id_cs[imesh],
                          id_var_match[imesh]);

    PDM_writer_var_write (id_cs[imesh],
                          id_var_cell_match[imesh]);

    PDM_writer_var_write (id_cs[imesh],
                          id_var_origin[imesh]);

    PDM_writer_var_write (id_cs[imesh],
                          id_var_field[imesh]);

    PDM_writer_var_free (id_cs[imesh],
                         id_var_num_part[imesh]);

    PDM_writer_var_free (id_cs[imesh],
                         id_var_match[imesh]);

    PDM_writer_var_free (id_cs[imesh],
                         id_var_cell_match[imesh]);

    PDM_writer_var_free (id_cs[imesh],
                         id_var_origin[imesh]);

    PDM_writer_var_free (id_cs[imesh],
                         id_var_field[imesh]);

    for (int ipart = 0; ipart < n_part; ipart++) {
      free (val_num_part[ipart]);
      free (val_match[ipart]);
      free (val_cell_match[ipart]);
      free (val_origin[ipart]);
    }

    free (val_num_part);
    free (val_match);
    free (val_cell_match);
    free (val_origin);

    PDM_writer_step_end (id_cs[imesh]);
    PDM_writer_geom_data_free (id_cs[imesh],
                               id_geom[imesh]);

    PDM_writer_geom_free (id_cs[imesh],
                          id_geom[imesh]);
    PDM_writer_free (id_cs[imesh]);

    free (_olface_nb);
    free (_olface_idx);
    free (debPartProcs);
  }

}

/**
 *
 * \brief  Main
 *
 */

int
main
(
 int argc,
 char *argv[]
 )
{
  PDM_MPI_Init (&argc, &argv);

  int numProcs;
  PDM_MPI_Comm_size (PDM_MPI_COMM_WORLD, &numProcs);

  assert (numProcs == 1);

  const int n_part = 1;

  int nFaceA = 210;
  int nVtxA = 242;
  int nFaceA_merge = 3 * nFaceA;
  int nVtxA_merge = 3 * nVtxA;

  /* int nFaceA = 1; */
  /* int faceVtxIdxA[2] = {0, 4}; */
  /* int faceVtxA[4] = {1, 2, 3, 4}; */
  /* PDM_g_num_t faceLNToGNA[1] = {1}; */
  /* int nVtxA = 4; */
  /* double vtxCoordA[12] = {-2.65000000e+00, -3.47962203e-01, -1.87872219e-03, */
  /*                         -2.65000000e+00, -3.47964146e-01, -1.47559256e-03, */
  /*                         -2.65000000e+00, -3.48385375e-01, -1.36252061e-03, */
  /*                         -2.65000000e+00, -3.48383562e-01, -1.76618816e-03}; */

  /* PDM_g_num_t vtxLNToGNA[4] = {1, 2, 3, 4}; */

  int faceVtxIdxA[211] = {0,4,8, 12, 16, 20, 24, 28, 32, 36, 40, 44, 48, 52, 56, 60, 64, 68,
    72, 76, 80, 84, 88, 92, 96,100,104,108,112,116,120,124,128,132,136,140,
    144,148,152,156,160,164,168,172,176,180,184,188,192,196,200,204,208,212,
    216,220,224,228,232,236,240,244,248,252,256,260,264,268,272,276,280,284,
    288,292,296,300,304,308,312,316,320,324,328,332,336,340,344,348,352,356,
    360,364,368,372,376,380,384,388,392,396,400,404,408,412,416,420,424,428,
    432,436,440,444,448,452,456,460,464,468,472,476,480,484,488,492,496,500,
    504,508,512,516,520,524,528,532,536,540,544,548,552,556,560,564,568,572,
    576,580,584,588,592,596,600,604,608,612,616,620,624,628,632,636,640,644,
    648,652,656,660,664,668,672,676,680,684,688,692,696,700,704,708,712,716,
    720,724,728,732,736,740,744,748,752,756,760,764,768,772,776,780,784,788,
    792,796,800,804,808,812,816,820,824,828,832,836,840};
  int faceVtxA[840] = {1,2,13,12,2,3,14,13,3,4,15,14,4,5,16,15,5,6,17,16,6,7,18,17,7,8,19,18,8,9,20,19,9,10,21,20,
    10,11,22,21,12,13,24,23,13,14,25,24,14,15,26,25,15,16,27,26,16,17,28,27,17,18,29,28,18,19,30,29,19,20,31,30,
    20,21,32,31,21,22,33,32,23,24,35,34,24,25,36,35,25,26,37,36,26,27,38,37,27,28,39,38,28,29,40,39,29,30,41,40,
    30,31,42,41,31,32,43,42,32,33,44,43,34,35,46,45,35,36,47,46,36,37,48,47,37,38,49,48,38,39,50,49,39,40,51,50,
    40,41,52,51,41,42,53,52,42,43,54,53,43,44,55,54,45,46,57,56,46,47,58,57,47,48,59,58,48,49,60,59,49,50,61,60,
    50,51,62,61,51,52,63,62,52,53,64,63,53,54,65,64,54,55,66,65,56,57,68,67,57,58,69,68,58,59,70,69,59,60,71,70,
    60,61,72,71,61,62,73,72,62,63,74,73,63,64,75,74,64,65,76,75,65,66,77,76,67,68,79,78,68,69,80,79,69,70,81,80,
    70,71,82,81,71,72,83,82,72,73,84,83,73,74,85,84,74,75,86,85,75,76,87,86,76,77,88,87,78,79,90,89,79,80,91,90,
    80,81,92,91,81,82,93,92,82,83,94,93,83,84,95,94,84,85,96,95,85,86,97,96,86,87,98,97,87,88,99,98,89,90,101,100,
    90,91,102,101,91,92,103,102,92,93,104,103,93,94,105,104,94,95,106,105,95,96,107,106,96,97,108,107,97,98,109,108,98,99,110,109,
    100,101,112,111,101,102,113,112,102,103,114,113,103,104,115,114,104,105,116,115,105,106,117,116,106,107,118,117,107,108,119,118,108,109,120,119,
    109,110,121,120,111,112,123,122,112,113,124,123,113,114,125,124,114,115,126,125,115,116,127,126,116,117,128,127,117,118,129,128,118,119,130,129,
    119,120,131,130,120,121,132,131,122,123,134,133,123,124,135,134,124,125,136,135,125,126,137,136,126,127,138,137,127,128,139,138,128,129,140,139,
    129,130,141,140,130,131,142,141,131,132,143,142,133,134,145,144,134,135,146,145,135,136,147,146,136,137,148,147,137,138,149,148,138,139,150,149,
    139,140,151,150,140,141,152,151,141,142,153,152,142,143,154,153,144,145,156,155,145,146,157,156,146,147,158,157,147,148,159,158,148,149,160,159,
    149,150,161,160,150,151,162,161,151,152,163,162,152,153,164,163,153,154,165,164,155,156,167,166,156,157,168,167,157,158,169,168,158,159,170,169,
    159,160,171,170,160,161,172,171,161,162,173,172,162,163,174,173,163,164,175,174,164,165,176,175,166,167,178,177,167,168,179,178,168,169,180,179,
    169,170,181,180,170,171,182,181,171,172,183,182,172,173,184,183,173,174,185,184,174,175,186,185,175,176,187,186,177,178,189,188,178,179,190,189,
    179,180,191,190,180,181,192,191,181,182,193,192,182,183,194,193,183,184,195,194,184,185,196,195,185,186,197,196,186,187,198,197,188,189,200,199,
    189,190,201,200,190,191,202,201,191,192,203,202,192,193,204,203,193,194,205,204,194,195,206,205,195,196,207,206,196,197,208,207,197,198,209,208,
    199,200,211,210,200,201,212,211,201,202,213,212,202,203,214,213,203,204,215,214,204,205,216,215,205,206,217,216,206,207,218,217,207,208,219,218,
    208,209,220,219,210,211,222,221,211,212,223,222,212,213,224,223,213,214,225,224,214,215,226,225,215,216,227,226,216,217,228,227,217,218,229,228,
    218,219,230,229,219,220,231,230,221,222,233,232,222,223,234,233,223,224,235,234,224,225,236,235,225,226,237,236,226,227,238,237,227,228,239,238,
    228,229,240,239,229,230,241,240,230,231,242,241};
  PDM_g_num_t faceLNToGNA[210];
  for (int i = 0; i < 210; ++i) {
    faceLNToGNA[i] = i+1;
  }
  double vtxCoordA[726] = {-2.65000000e+00,-2.32723059e-01,-4.13913703e-02,-2.65000000e+00
    ,-2.34025276e-01,-3.32481284e-02,-2.65000000e+00,-2.35028639e-01
    ,-2.51953664e-02,-2.65000000e+00,-2.35734061e-01,-1.73988545e-02
    ,-2.65000000e+00,-2.36167763e-01,-9.90228843e-03,-2.65000000e+00
    ,-2.36361164e-01,-2.58223080e-03,-2.65000000e+00,-2.36327014e-01
    ,4.77598706e-03,-2.65000000e+00,-2.36050349e-01,1.23895271e-02
    ,-2.65000000e+00,-2.35499581e-01,2.03276935e-02,-2.65000000e+00
    ,-2.34655815e-01,2.84590261e-02,-2.65000000e+00,-2.34123419e-01
    ,3.25498478e-02,-2.65000000e+00,-2.33683369e-01,-4.15621679e-02
    ,-2.65000000e+00,-2.34990959e-01,-3.33853237e-02,-2.65000000e+00
    ,-2.35998463e-01,-2.52993327e-02,-2.65000000e+00,-2.36706796e-01
    ,-1.74706492e-02,-2.65000000e+00,-2.37142287e-01,-9.94314929e-03
    ,-2.65000000e+00,-2.37336486e-01,-2.59288613e-03,-2.65000000e+00
    ,-2.37302196e-01,4.79569472e-03,-2.65000000e+00,-2.37024389e-01
    ,1.24406513e-02,-2.65000000e+00,-2.36471348e-01,2.04115738e-02
    ,-2.65000000e+00,-2.35624100e-01,2.85764596e-02,-2.65000000e+00
    ,-2.35089507e-01,3.26841617e-02,-2.65000000e+00,-2.37460795e-01
    ,-3.94191862e-02,-2.65000000e+00,-2.38695906e-01,-3.10767800e-02
    ,-2.65000000e+00,-2.39623815e-01,-2.28457651e-02,-2.65000000e+00
    ,-2.40248698e-01,-1.49018311e-02,-2.65000000e+00,-2.40600164e-01
    ,-7.28441549e-03,-2.65000000e+00,-2.40710370e-01,1.39540482e-04
    ,-2.65000000e+00,-2.40590452e-01,7.59843603e-03,-2.65000000e+00
    ,-2.40221913e-01,1.53275615e-02,-2.65000000e+00,-2.39569308e-01
    ,2.34104264e-02,-2.65000000e+00,-2.38612541e-01,3.17105165e-02
    ,-2.65000000e+00,-2.38019835e-01,3.58895474e-02,-2.65000000e+00
    ,-2.43686930e-01,-3.68414344e-02,-2.65000000e+00,-2.44832945e-01
    ,-2.82389870e-02,-2.65000000e+00,-2.45661325e-01,-1.97768765e-02
    ,-2.65000000e+00,-2.46180899e-01,-1.16437254e-02,-2.65000000e+00
    ,-2.46425683e-01,-3.87220841e-03,-2.65000000e+00,-2.46428493e-01
    ,3.68905873e-03,-2.65000000e+00,-2.46197519e-01,1.12868402e-02
    ,-2.65000000e+00,-2.45709184e-01,1.91731153e-02,-2.65000000e+00
    ,-2.44922882e-01,2.74480111e-02,-2.65000000e+00,-2.43817037e-01
    ,3.59703215e-02,-2.65000000e+00,-2.43144672e-01,4.02651166e-02
    ,-2.65000000e+00,-2.54021503e-01,-3.39645556e-02,-2.65000000e+00
    ,-2.55066630e-01,-2.49304893e-02,-2.65000000e+00,-2.55777998e-01
    ,-1.60664418e-02,-2.65000000e+00,-2.56169423e-01,-7.59879117e-03
    ,-2.65000000e+00,-2.56281707e-01,4.48834375e-04,-2.65000000e+00
    ,-2.56149060e-01,8.25676104e-03,-2.65000000e+00,-2.55775832e-01
    ,1.61008876e-02,-2.65000000e+00,-2.55131573e-01,2.42568633e-02
    ,-2.65000000e+00,-2.54169151e-01,3.28414038e-02,-2.65000000e+00
    ,-2.52864702e-01,4.17127999e-02,-2.65000000e+00,-2.52084431e-01
    ,4.61947494e-02,-2.65000000e+00,-2.67970176e-01,-3.05417766e-02
    ,-2.65000000e+00,-2.68892349e-01,-2.09217612e-02,-2.65000000e+00
    ,-2.69459429e-01,-1.15078813e-02,-2.65000000e+00,-2.69692658e-01
    ,-2.58560593e-03,-2.65000000e+00,-2.69641883e-01,5.83697263e-03
    ,-2.65000000e+00,-2.69342382e-01,1.39820101e-02,-2.65000000e+00
    ,-2.68792739e-01,2.21648080e-02,-2.65000000e+00,-2.67953271e-01
    ,3.06897324e-02,-2.65000000e+00,-2.66768604e-01,3.96903919e-02
    ,-2.65000000e+00,-2.65211585e-01,4.90268340e-02,-2.65000000e+00
    ,-2.64292710e-01,5.37603827e-02,-2.65000000e+00,-2.81883459e-01
    ,-2.69572492e-02,-2.65000000e+00,-2.82672181e-01,-1.67754497e-02
    ,-2.65000000e+00,-2.83086696e-01,-6.84836562e-03,-2.65000000e+00
    ,-2.83158442e-01,2.50481838e-03,-2.65000000e+00,-2.82944144e-01
    ,1.12955241e-02,-2.65000000e+00,-2.82477413e-01,1.97860773e-02
    ,-2.65000000e+00,-2.81749496e-01,2.83231161e-02,-2.65000000e+00
    ,-2.80711162e-01,3.72319915e-02,-2.65000000e+00,-2.79297686e-01
    ,4.66666891e-02,-2.65000000e+00,-2.77478929e-01,5.64838146e-02
    ,-2.65000000e+00,-2.76417348e-01,6.14689116e-02,-2.65000000e+00
    ,-2.95740207e-01,-2.30098609e-02,-2.65000000e+00,-2.96380252e-01
    ,-1.22666171e-02,-2.65000000e+00,-2.96628364e-01,-1.82676760e-03
    ,-2.65000000e+00,-2.96527351e-01,7.95319466e-03,-2.65000000e+00
    ,-2.96140147e-01,1.71095594e-02,-2.65000000e+00,-2.95497024e-01
    ,2.59467246e-02,-2.65000000e+00,-2.94580934e-01,3.48395867e-02
    ,-2.65000000e+00,-2.93333121e-01,4.41293931e-02,-2.65000000e+00
    ,-2.91679633e-01,5.39881019e-02,-2.65000000e+00,-2.89587068e-01
    ,6.42732739e-02,-2.65000000e+00,-2.88376089e-01,6.95050716e-02
    ,-2.65000000e+00,-3.09531368e-01,-1.87452856e-02,-2.65000000e+00
    ,-3.10009188e-01,-7.44018969e-03,-2.65000000e+00,-3.10078559e-01
    ,3.51290932e-03,-2.65000000e+00,-3.09794948e-01,1.37165413e-02
    ,-2.65000000e+00,-3.09226615e-01,2.32369022e-02,-2.65000000e+00
    ,-3.08399000e-01,3.24208277e-02,-2.65000000e+00,-3.07286139e-01
    ,4.16687180e-02,-2.65000000e+00,-3.05819830e-01,5.13350266e-02
    ,-2.65000000e+00,-3.03917018e-01,6.16076234e-02,-2.65000000e+00
    ,-3.01540588e-01,7.23486481e-02,-2.65000000e+00,-3.00174436e-01
    ,7.78226269e-02,-2.65000000e+00,-3.23240473e-01,-1.44417366e-02
    ,-2.65000000e+00,-3.23552641e-01,-2.57974541e-03,-2.65000000e+00
    ,-3.23441114e-01,8.87765609e-03,-2.65000000e+00,-3.22974993e-01
    ,1.94966887e-02,-2.65000000e+00,-3.22226578e-01,2.93768481e-02
    ,-2.65000000e+00,-3.21215291e-01,3.89063469e-02,-2.65000000e+00
    ,-3.19905983e-01,4.85090571e-02,-2.65000000e+00,-3.18221030e-01
    ,5.85520533e-02,-2.65000000e+00,-3.16067865e-01,6.92392350e-02
    ,-2.65000000e+00,-3.13405371e-01,8.04365611e-02,-2.65000000e+00
    ,-3.11882700e-01,8.61518916e-02,-2.65000000e+00,-3.36862525e-01
    ,-1.05405600e-02,-2.65000000e+00,-3.37022157e-01,1.87891496e-03
    ,-2.65000000e+00,-3.36743137e-01,1.38392178e-02,-2.65000000e+00
    ,-3.36108543e-01,2.48698933e-02,-2.65000000e+00,-3.35194011e-01
    ,3.51061176e-02,-2.65000000e+00,-3.34012628e-01,4.49780938e-02
    ,-2.65000000e+00,-3.32520244e-01,5.49340653e-02,-2.65000000e+00
    ,-3.30630181e-01,6.53540183e-02,-2.65000000e+00,-3.28240447e-01
    ,7.64570042e-02,-2.65000000e+00,-3.25305326e-01,8.81130463e-02
    ,-2.65000000e+00,-3.23632512e-01,9.40715768e-02,-2.65000000e+00
    ,-3.50415545e-01,-7.31376407e-03,-2.65000000e+00,-3.50446269e-01
    ,5.65317372e-03,-2.65000000e+00,-3.50023906e-01,1.81055497e-02
    ,-2.65000000e+00,-3.49244509e-01,2.95435022e-02,-2.65000000e+00
    ,-3.48185851e-01,4.01392428e-02,-2.65000000e+00,-3.46854724e-01
    ,5.03621503e-02,-2.65000000e+00,-3.45199086e-01,6.06806103e-02
    ,-2.65000000e+00,-3.43124683e-01,7.14842470e-02,-2.65000000e+00
    ,-3.40520602e-01,8.30076224e-02,-2.65000000e+00,-3.37335976e-01
    ,9.51261525e-02,-2.65000000e+00,-3.35525065e-01,1.01328556e-01
    ,-2.65000000e+00,-3.63927984e-01,-4.54232474e-03,-2.65000000e+00
    ,-3.63846154e-01,8.95468976e-03,-2.65000000e+00,-3.63298094e-01
    ,2.18793454e-02,-2.65000000e+00,-3.62391144e-01,3.37174923e-02
    ,-2.65000000e+00,-3.61203981e-01,4.46754335e-02,-2.65000000e+00
    ,-3.59737022e-01,5.52583552e-02,-2.65000000e+00,-3.57930973e-01
    ,6.59517188e-02,-2.65000000e+00,-3.55685268e-01,7.71505083e-02
    ,-2.65000000e+00,-3.52880375e-01,8.91047228e-02,-2.65000000e+00
    ,-3.49460419e-01,1.01693787e-01,-2.65000000e+00,-3.47519737e-01
    ,1.08139924e-01,-2.65000000e+00,-3.77417343e-01,-1.61505783e-03
    ,-2.65000000e+00,-3.77216193e-01,1.24259094e-02,-2.65000000e+00
    ,-3.76534705e-01,2.58471505e-02,-2.65000000e+00,-3.75492876e-01
    ,3.80993368e-02,-2.65000000e+00,-3.74169832e-01,4.94307235e-02
    ,-2.65000000e+00,-3.72559824e-01,6.03791133e-02,-2.65000000e+00
    ,-3.70597251e-01,7.14432438e-02,-2.65000000e+00,-3.68175868e-01
    ,8.30240293e-02,-2.65000000e+00,-3.65169414e-01,9.53821691e-02
    ,-2.65000000e+00,-3.61515687e-01,1.08410643e-01,-2.65000000e+00
    ,-3.59444304e-01,1.15092362e-01,-2.65000000e+00,-3.90883394e-01
    ,1.21009805e-03,-2.65000000e+00,-3.90565632e-01,1.58043978e-02
    ,-2.65000000e+00,-3.89752342e-01,2.97389283e-02,-2.65000000e+00
    ,-3.88577290e-01,4.24144063e-02,-2.65000000e+00,-3.87119699e-01
    ,5.41260662e-02,-2.65000000e+00,-3.85367862e-01,6.54438949e-02
    ,-2.65000000e+00,-3.83250624e-01,7.68781597e-02,-2.65000000e+00
    ,-3.80657022e-01,8.88342466e-02,-2.65000000e+00,-3.77455786e-01
    ,1.01579631e-01,-2.65000000e+00,-3.73577138e-01,1.15027882e-01
    ,-2.65000000e+00,-3.71378258e-01,1.21940483e-01,-2.65000000e+00
    ,-4.04334792e-01,3.47633036e-03,-2.65000000e+00,-4.03922538e-01
    ,1.85820409e-02,-2.65000000e+00,-4.03003100e-01,3.29728683e-02
    ,-2.65000000e+00,-4.01719214e-01,4.60476066e-02,-2.65000000e+00
    ,-4.00148552e-01,5.81364325e-02,-2.65000000e+00,-3.98273743e-01
    ,6.98336155e-02,-2.65000000e+00,-3.96018788e-01,8.16567689e-02
    ,-2.65000000e+00,-3.93268311e-01,9.40145966e-02,-2.65000000e+00
    ,-3.89883469e-01,1.07189501e-01,-2.65000000e+00,-3.85788965e-01
    ,1.21101541e-01,-2.65000000e+00,-3.83470993e-01,1.28252509e-01
    ,-2.65000000e+00,-4.17783487e-01,5.06623176e-03,-2.65000000e+00
    ,-4.17302098e-01,2.06801311e-02,-2.65000000e+00,-4.16301925e-01
    ,3.55164174e-02,-2.65000000e+00,-4.14932882e-01,4.89837969e-02
    ,-2.65000000e+00,-4.13271455e-01,6.14443957e-02,-2.65000000e+00
    ,-4.11295355e-01,7.35176154e-02,-2.65000000e+00,-4.08923981e-01
    ,8.57314825e-02,-2.65000000e+00,-4.06037593e-01,9.84996554e-02
    ,-2.65000000e+00,-4.02488837e-01,1.12122454e-01,-2.65000000e+00
    ,-3.98197024e-01,1.26522089e-01,-2.65000000e+00,-3.95770156e-01
    ,1.33920470e-01,-2.65000000e+00,-4.24666706e-01,5.70138439e-03
    ,-2.65000000e+00,-4.24155802e-01,2.15910282e-02,-2.65000000e+00
    ,-4.23118645e-01,3.66732759e-02,-2.65000000e+00,-4.21710065e-01
    ,5.03481614e-02,-2.65000000e+00,-4.20006670e-01,6.29977295e-02
    ,-2.65000000e+00,-4.17983934e-01,7.52578751e-02,-2.65000000e+00
    ,-4.15558809e-01,8.76652323e-02,-2.65000000e+00,-4.12609344e-01
    ,1.00637199e-01,-2.65000000e+00,-4.08984521e-01,1.14481343e-01
    ,-2.65000000e+00,-4.04599930e-01,1.29124797e-01,-2.65000000e+00
    ,-4.02120213e-01,1.36651568e-01,-2.65000000e+00,-4.26059228e-01
    ,5.83206672e-03,-2.65000000e+00,-4.25542266e-01,2.17774831e-02
    ,-2.65000000e+00,-4.24497549e-01,3.69094783e-02,-2.65000000e+00
    ,-4.23080907e-01,5.06263325e-02,-2.65000000e+00,-4.21368961e-01
    ,6.33141204e-02,-2.65000000e+00,-4.19336730e-01,7.56120728e-02
    ,-2.65000000e+00,-4.16900669e-01,8.80585647e-02,-2.65000000e+00
    ,-4.13938378e-01,1.01071749e-01,-2.65000000e+00,-4.10298093e-01
    ,1.14960663e-01,-2.65000000e+00,-4.05894650e-01,1.29653430e-01
    ,-2.65000000e+00,-4.03404198e-01,1.37206167e-01,-2.65000000e+00
    ,-4.26340964e-01,5.85859592e-03,-2.65000000e+00,-4.25822772e-01
    ,2.18152957e-02,-2.65000000e+00,-4.24776523e-01,3.69573553e-02
    ,-2.65000000e+00,-4.23358246e-01,5.06827002e-02,-2.65000000e+00
    ,-4.21644568e-01,6.33782203e-02,-2.65000000e+00,-4.19610414e-01
    ,7.56838214e-02,-2.65000000e+00,-4.17172138e-01,8.81382306e-02
    ,-2.65000000e+00,-4.14207249e-01,1.01159754e-01,-2.65000000e+00
    ,-4.10563833e-01,1.15057725e-01,-2.65000000e+00,-4.06156573e-01
    ,1.29760469e-01,-2.65000000e+00,-4.03663946e-01,1.37318459e-01
    ,-2.65000000e+00,-4.27115030e-01,5.91702041e-03,-2.65000000e+00
    ,-4.26593786e-01,2.19089324e-02,-2.65000000e+00,-4.25543196e-01
    ,3.70843351e-02,-2.65000000e+00,-4.24120263e-01,5.08356347e-02
    ,-2.65000000e+00,-4.22401657e-01,6.35539137e-02,-2.65000000e+00
    ,-4.20362192e-01,7.58807455e-02,-2.65000000e+00,-4.17918155e-01
    ,8.83553938e-02,-2.65000000e+00,-4.14946835e-01,1.01397159e-01
    ,-2.65000000e+00,-4.11296192e-01,1.15315664e-01,-2.65000000e+00
    ,-4.06880200e-01,1.30041386e-01,-2.65000000e+00,-4.04382072e-01
    ,1.37613224e-01,-2.65000000e+00,-4.27133168e-01,5.91838570e-03
    ,-2.65000000e+00,-4.26611853e-01,2.19111245e-02,-2.65000000e+00
    ,-4.25561161e-01,3.70873108e-02,-2.65000000e+00,-4.24138119e-01
    ,5.08392196e-02,-2.65000000e+00,-4.22419397e-01,6.35580324e-02
    ,-2.65000000e+00,-4.20379807e-01,7.58853618e-02,-2.65000000e+00
    ,-4.17935635e-01,8.83604837e-02,-2.65000000e+00,-4.14964165e-01
    ,1.01402722e-01,-2.65000000e+00,-4.11313353e-01,1.15321707e-01
    ,-2.65000000e+00,-4.06897158e-01,1.30047965e-01,-2.65000000e+00
    ,-4.04398901e-01,1.37620127e-01};

  double coeffrand = 1e-7;
  for (int i = 0; i < 242; i++) {
    vtxCoordA[2*i] += coeffrand * random01();
  }


  PDM_g_num_t vtxLNToGNA[242];
  for (int i = 0; i < 242; ++i) {
    vtxLNToGNA[i] = i+1;
  }


  int *faceVtxIdxA_merge = malloc (sizeof(int) * (nFaceA_merge + 1));
  int *faceVtxA_merge = malloc (sizeof(int) * 4 * nFaceA_merge);
  PDM_g_num_t *faceLNToGNA_merge = malloc (sizeof(PDM_g_num_t) * 4 * nFaceA_merge);

  double *vtxCoordA_merge = malloc (sizeof(double) * 3 * nVtxA_merge);
  PDM_g_num_t *vtxLNToGNA_merge = malloc (sizeof(PDM_g_num_t) * nVtxA_merge);

  faceVtxIdxA_merge[0] = 0;
  for (int i = 0; i < nFaceA_merge; i++) {
    faceVtxIdxA_merge[i+1] = faceVtxIdxA_merge[i] + 4;
  }

  for (int i = 0; i < faceVtxIdxA_merge[nFaceA]; ++i) {
    faceVtxA_merge[i]                           = faceVtxA[i];
    faceVtxA_merge[i+faceVtxIdxA_merge[nFaceA]] = faceVtxA[i] + nVtxA;
    faceVtxA_merge[i+2*faceVtxIdxA_merge[nFaceA]] = faceVtxA[i] + 2*nVtxA;
  }

  for (int i = 0; i < nFaceA; ++i) {
    faceLNToGNA_merge[i] = i+1;
    faceLNToGNA_merge[nFaceA+i] = nFaceA+i+1;
    faceLNToGNA_merge[2*nFaceA+i] = 2*nFaceA+i+1;
  }

  for (int i = 0; i < nVtxA; ++i) {
    vtxLNToGNA_merge[i] = i+1;
    vtxLNToGNA_merge[nVtxA+i] = nVtxA+i+1;
    vtxLNToGNA_merge[2*nVtxA+i] = 2*nVtxA+i+1;
  }

  double teta = 18.0*PDM_PI/180.;
  for (int i = 0; i < nVtxA; ++i) {
    vtxCoordA_merge[3*i]           = vtxCoordA[3*i];
    vtxCoordA_merge[3*(i+nVtxA)]   = vtxCoordA[3*i];
    vtxCoordA_merge[3*(i+2*nVtxA)] = vtxCoordA[3*i];

    vtxCoordA_merge[3*i+1]           = vtxCoordA[3*i+1];
    vtxCoordA_merge[3*(i+nVtxA)+1]   = vtxCoordA[3*i+1] * cos(-teta) +
      vtxCoordA[3*i+2] * sin(-teta);
    vtxCoordA_merge[3*(i+2*nVtxA)+1]   = vtxCoordA[3*i+1] * cos(teta) +
      vtxCoordA[3*i+2] * sin(teta);

    vtxCoordA_merge[3*i+2]           = vtxCoordA[3*i+2];
    vtxCoordA_merge[3*(i+nVtxA)+2]   = - vtxCoordA[3*i+1] * sin(-teta) +
      vtxCoordA[3*i+2] * cos(-teta);
    vtxCoordA_merge[3*(i+2*nVtxA)+2]  = - vtxCoordA[3*i+1] * sin(teta) +
      vtxCoordA[3*i+2] * cos(teta);
  }

  int id_gnum = PDM_gnum_create (3, 1, PDM_TRUE, 1e-3, PDM_MPI_COMM_WORLD);

  double *char_length = malloc(sizeof(double) * nVtxA_merge);

  for (int i = 0; i < nVtxA_merge; i++) {
    char_length[i] = 1.e-3;
  }

  PDM_gnum_set_from_coords (id_gnum, 0, nVtxA_merge, vtxCoordA_merge, char_length);

  PDM_gnum_compute (id_gnum);

  PDM_g_num_t *new_gnum = PDM_gnum_get (id_gnum, 0);

  int _nVtxA_merge = 0;
  for (int i = 0; i < nVtxA_merge; i++) {
    _nVtxA_merge = PDM_MAX (_nVtxA_merge, new_gnum[i]);
  }

  printf("_nVtxA_merge, nVtxA_merge : %d %d\n", _nVtxA_merge, nVtxA_merge);

  double *_vtxCoordA_merge = malloc (sizeof(double) * 3 * _nVtxA_merge);
  PDM_g_num_t *_vtxLNToGNA_merge = malloc (sizeof(PDM_g_num_t) * _nVtxA_merge);

  /* int *_faceVtxIdxA_merge = malloc (sizeof(int) * (nFaceA_merge + 1)); */
  int *_faceVtxA_merge = malloc (sizeof(int) * 4 * nFaceA_merge);
  /* PDM_g_num_t *faceLNToGNA_merge = malloc (sizeof(PDM_g_num_t) * 4 * nFaceA_merge); */

  for (int i = 0; i < nVtxA_merge; i++) {
    for (int j = 0; j < 3; j++) {
      _vtxCoordA_merge[3*(new_gnum[i]-1)+j] = vtxCoordA_merge[3*i+j];
    }
  }

  for (int i = 0; i < _nVtxA_merge; i++) {
    _vtxLNToGNA_merge[i] = i + 1;
  }

  for (int i = 0; i < faceVtxIdxA_merge[nFaceA_merge]; i++) {
    _faceVtxA_merge[i] = new_gnum[faceVtxA_merge[i]-1];
  }

  PDM_gnum_free (id_gnum, 0);

  free (char_length);
  free (faceVtxA_merge);
  free (vtxCoordA_merge);
  free (vtxLNToGNA_merge);

  faceVtxA_merge = _faceVtxA_merge;
  vtxCoordA_merge = _vtxCoordA_merge;
  vtxLNToGNA_merge = _vtxLNToGNA_merge;

  nVtxA_merge = _nVtxA_merge;
  /* printf("faceVtx : \n"); */
  /* for (int i = 0; i < nFaceA; i++) { */
  /*   for (int j = faceVtxIdxA_merge[i]; j < faceVtxIdxA_merge[i+1]; j++) { */
  /*     printf ("%d ",faceVtxA_merge[j] ); */
  /*   } */
  /*   printf(" / "); */
  /*   for (int j = faceVtxIdxA_merge[nFaceA+i]; j < faceVtxIdxA_merge[nFaceA +i+1]; j++) { */
  /*     printf ("%d ",faceVtxA_merge[j] ); */
  /*   } */
  /*   printf(" / "); */
  /*   for (int j = faceVtxIdxA_merge[2*nFaceA+i]; j < faceVtxIdxA_merge[2*nFaceA+i+1]; j++) { */
  /*     printf ("%d ",faceVtxA_merge[j] ); */
  /*   } */
  /*   printf("\n"); */
  /* } */

  int nFaceB = 200;
  int faceVtxIdxB[201] = {0,4,8, 12, 16, 20, 24, 28, 32, 36, 40, 44, 48, 52, 56, 60, 64, 68,
    72, 76, 80, 84, 88, 92, 96,100,104,108,112,116,120,124,128,132,136,140,
    144,148,152,156,160,164,168,172,176,180,184,188,192,196,200,204,208,212,
    216,220,224,228,232,236,240,244,248,252,256,260,264,268,272,276,280,284,
    288,292,296,300,304,308,312,316,320,324,328,332,336,340,344,348,352,356,
    360,364,368,372,376,380,384,388,392,396,400,404,408,412,416,420,424,428,
    432,436,440,444,448,452,456,460,464,468,472,476,480,484,488,492,496,500,
    504,508,512,516,520,524,528,532,536,540,544,548,552,556,560,564,568,572,
    576,580,584,588,592,596,600,604,608,612,616,620,624,628,632,636,640,644,
    648,652,656,660,664,668,672,676,680,684,688,692,696,700,704,708,712,716,
    720,724,728,732,736,740,744,748,752,756,760,764,768,772,776,780,784,788,
    792,796,800};

  int faceVtxB[800] = {1,2,13,12,2,3,14,13,3,4,15,14,4,5,16,15,5,6
    ,17,16,6,7,18,17,7,8,19,18,8,9,20,19,9,10,21,20
    ,10,11,22,21,12,13,24,23,13,14,25,24,14,15,26,25,15,16
    ,27,26,16,17,28,27,17,18,29,28,18,19,30,29,19,20,31,30
    ,20,21,32,31,21,22,33,32,23,24,35,34,24,25,36,35,25,26
    ,37,36,26,27,38,37,27,28,39,38,28,29,40,39,29,30,41,40
    ,30,31,42,41,31,32,43,42,32,33,44,43,34,35,46,45,35,36
    ,47,46,36,37,48,47,37,38,49,48,38,39,50,49,39,40,51,50
    ,40,41,52,51,41,42,53,52,42,43,54,53,43,44,55,54,45,46
    ,57,56,46,47,58,57,47,48,59,58,48,49,60,59,49,50,61,60
    ,50,51,62,61,51,52,63,62,52,53,64,63,53,54,65,64,54,55
    ,66,65,56,57,68,67,57,58,69,68,58,59,70,69,59,60,71,70
    ,60,61,72,71,61,62,73,72,62,63,74,73,63,64,75,74,64,65
    ,76,75,65,66,77,76,67,68,79,78,68,69,80,79,69,70,81,80
    ,70,71,82,81,71,72,83,82,72,73,84,83,73,74,85,84,74,75
    ,86,85,75,76,87,86,76,77,88,87,78,79,90,89,79,80,91,90
    ,80,81,92,91,81,82,93,92,82,83,94,93,83,84,95,94,84,85
    ,96,95,85,86,97,96,86,87,98,97,87,88,99,98,89,90,101,100
    ,90,91,102,101,91,92,103,102,92,93,104,103,93,94,105,104,94,95
    ,106,105,95,96,107,106,96,97,108,107,97,98,109,108,98,99,110,109
    ,100,101,112,111,101,102,113,112,102,103,114,113,103,104,115,114,104,105
    ,116,115,105,106,117,116,106,107,118,117,107,108,119,118,108,109,120,119
    ,109,110,121,120,111,112,123,122,112,113,124,123,113,114,125,124,114,115
    ,126,125,115,116,127,126,116,117,128,127,117,118,129,128,118,119,130,129
    ,119,120,131,130,120,121,132,131,122,123,134,133,123,124,135,134,124,125
    ,136,135,125,126,137,136,126,127,138,137,127,128,139,138,128,129,140,139
    ,129,130,141,140,130,131,142,141,131,132,143,142,133,134,145,144,134,135
    ,146,145,135,136,147,146,136,137,148,147,137,138,149,148,138,139,150,149
    ,139,140,151,150,140,141,152,151,141,142,153,152,142,143,154,153,144,145
    ,156,155,145,146,157,156,146,147,158,157,147,148,159,158,148,149,160,159
    ,149,150,161,160,150,151,162,161,151,152,163,162,152,153,164,163,153,154
    ,165,164,155,156,167,166,156,157,168,167,157,158,169,168,158,159,170,169
    ,159,160,171,170,160,161,172,171,161,162,173,172,162,163,174,173,163,164
    ,175,174,164,165,176,175,166,167,178,177,167,168,179,178,168,169,180,179
    ,169,170,181,180,170,171,182,181,171,172,183,182,172,173,184,183,173,174
    ,185,184,174,175,186,185,175,176,187,186,177,178,189,188,178,179,190,189
    ,179,180,191,190,180,181,192,191,181,182,193,192,182,183,194,193,183,184
    ,195,194,184,185,196,195,185,186,197,196,186,187,198,197,188,189,200,199
    ,189,190,201,200,190,191,202,201,191,192,203,202,192,193,204,203,193,194
    ,205,204,194,195,206,205,195,196,207,206,196,197,208,207,197,198,209,208
    ,199,200,211,210,200,201,212,211,201,202,213,212,202,203,214,213,203,204
    ,215,214,204,205,216,215,205,206,217,216,206,207,218,217,207,208,219,218
    ,208,209,220,219,210,211,222,221,211,212,223,222,212,213,224,223,213,214
    ,225,224,214,215,226,225,215,216,227,226,216,217,228,227,217,218,229,228
    ,218,219,230,229,219,220,231,230};
  PDM_g_num_t faceLNToGNB[200];
  for (int i = 0; i < 200; ++i) {
    faceLNToGNB[i] = i+1;
  }
  int nVtxB = 231;
  double vtxCoordB[693] = {-2.65000000e+00,-2.36360365e-01,2.65441202e-03,-2.65000000e+00
    ,-2.36164234e-01,9.98610683e-03,-2.65000000e+00,-2.35735047e-01
    ,1.73854990e-02,-2.65000000e+00,-2.35021648e-01,2.52604941e-02
    ,-2.65000000e+00,-2.34134867e-01,3.24674007e-02,-2.65000000e+00
    ,-2.33035135e-01,3.95966390e-02,-2.65000000e+00,-2.31694489e-01
    ,4.68073869e-02,-2.65000000e+00,-2.30113066e-01,5.40485406e-02
    ,-2.65000000e+00,-2.28176530e-01,6.17149814e-02,-2.65000000e+00
    ,-2.26173257e-01,6.86944357e-02,-2.65000000e+00,-2.23971806e-01
    ,7.55638653e-02,-2.65000000e+00,-2.36592431e-01,2.63144237e-03
    ,-2.65000000e+00,-2.36396902e-01,9.97032939e-03,-2.65000000e+00
    ,-2.35968052e-01,1.73776037e-02,-2.65000000e+00,-2.35255027e-01
    ,2.52581706e-02,-2.65000000e+00,-2.34368104e-01,3.24729813e-02
    ,-2.65000000e+00,-2.33267937e-01,3.96102553e-02,-2.65000000e+00
    ,-2.31926761e-01,4.68282010e-02,-2.65000000e+00,-2.30344438e-01
    ,5.40771926e-02,-2.65000000e+00,-2.28407398e-01,6.17491948e-02
    ,-2.65000000e+00,-2.26402750e-01,6.87364339e-02,-2.65000000e+00
    ,-2.24199612e-01,7.56137322e-02,-2.65000000e+00,-2.37598411e-01
    ,2.53128929e-03,-2.65000000e+00,-2.37405508e-01,9.90136497e-03
    ,-2.65000000e+00,-2.36978140e-01,1.73428124e-02,-2.65000000e+00
    ,-2.36266748e-01,2.52474908e-02,-2.65000000e+00,-2.35379236e-01
    ,3.24965779e-02,-2.65000000e+00,-2.34277199e-01,3.96687052e-02
    ,-2.65000000e+00,-2.32933737e-01,4.69178663e-02,-2.65000000e+00
    ,-2.31347533e-01,5.42008422e-02,-2.65000000e+00,-2.29408335e-01
    ,6.18969147e-02,-2.65000000e+00,-2.27397742e-01,6.89179159e-02
    ,-2.65000000e+00,-2.25187305e-01,7.58293459e-02,-2.65000000e+00
    ,-2.41958835e-01,2.08623163e-03,-2.65000000e+00,-2.41777645e-01
    ,9.59170569e-03,-2.65000000e+00,-2.41357061e-01,1.71813709e-02
    ,-2.65000000e+00,-2.40653080e-01,2.51897902e-02,-2.65000000e+00
    ,-2.39763369e-01,3.25876836e-02,-2.65000000e+00,-2.38653562e-01
    ,3.99112476e-02,-2.65000000e+00,-2.37300487e-01,4.72959724e-02
    ,-2.65000000e+00,-2.35697796e-01,5.47264024e-02,-2.65000000e+00
    ,-2.33749690e-01,6.25260979e-02,-2.65000000e+00,-2.31713647e-01
    ,6.96937296e-02,-2.65000000e+00,-2.29471846e-01,7.67535162e-02
    ,-2.65000000e+00,-2.54496637e-01,1.21418535e-03,-2.65000000e+00
    ,-2.54336000e-01,9.12204783e-03,-2.65000000e+00,-2.53922448e-01
    ,1.71290061e-02,-2.65000000e+00,-2.53225282e-01,2.54355819e-02
    ,-2.65000000e+00,-2.52317438e-01,3.32554217e-02,-2.65000000e+00
    ,-2.51173421e-01,4.10112820e-02,-2.65000000e+00,-2.49777699e-01
    ,4.87966563e-02,-2.65000000e+00,-2.48116674e-01,5.66403419e-02
    ,-2.65000000e+00,-2.46128657e-01,6.47355893e-02,-2.65000000e+00
    ,-2.44008697e-01,7.23171357e-02,-2.65000000e+00,-2.41665481e-01
    ,7.97985447e-02,-2.65000000e+00,-2.67936888e-01,9.84408412e-04
    ,-2.65000000e+00,-2.67776144e-01,9.33173543e-03,-2.65000000e+00
    ,-2.67348973e-01,1.77671502e-02,-2.65000000e+00,-2.66632972e-01
    ,2.64197454e-02,-2.65000000e+00,-2.65684860e-01,3.46799633e-02
    ,-2.65000000e+00,-2.64484145e-01,4.28868524e-02,-2.65000000e+00
    ,-2.63019571e-01,5.11062642e-02,-2.65000000e+00,-2.61278069e-01
    ,5.93710006e-02,-2.65000000e+00,-2.59217319e-01,6.78050609e-02
    ,-2.65000000e+00,-2.56988697e-01,7.58152644e-02,-2.65000000e+00
    ,-2.54518924e-01,8.37332797e-02,-2.65000000e+00,-2.81376801e-01
    ,7.71428629e-04,-2.65000000e+00,-2.81215402e-01,9.56018327e-03
    ,-2.65000000e+00,-2.80774660e-01,1.84143913e-02,-2.65000000e+00
    ,-2.80036804e-01,2.74387978e-02,-2.65000000e+00,-2.79048946e-01
    ,3.61273431e-02,-2.65000000e+00,-2.77791908e-01,4.47789604e-02
    ,-2.65000000e+00,-2.76257632e-01,5.34342625e-02,-2.65000000e+00
    ,-2.74437212e-01,6.21105133e-02,-2.65000000e+00,-2.72296713e-01
    ,7.09083884e-02,-2.65000000e+00,-2.69961829e-01,7.93354282e-02
    ,-2.65000000e+00,-2.67366856e-01,8.76838856e-02,-2.65000000e+00
    ,-2.94816677e-01,4.50903044e-04,-2.65000000e+00,-2.94657982e-01
    ,9.68245186e-03,-2.65000000e+00,-2.94207251e-01,1.89517663e-02
    ,-2.65000000e+00,-2.93450049e-01,2.83574474e-02,-2.65000000e+00
    ,-2.92426268e-01,3.74693763e-02,-2.65000000e+00,-2.91116457e-01
    ,4.65648408e-02,-2.65000000e+00,-2.89515584e-01,5.56579086e-02
    ,-2.65000000e+00,-2.87620362e-01,6.47425926e-02,-2.65000000e+00
    ,-2.85401110e-01,7.39140216e-02,-2.65000000e+00,-2.82964513e-01
    ,8.27536129e-02,-2.65000000e+00,-2.80247985e-01,9.15321976e-02
    ,-2.65000000e+00,-3.08256175e-01,7.66897992e-05,-2.65000000e+00
    ,-3.08101930e-01,9.75066664e-03,-2.65000000e+00,-3.07642996e-01
    ,1.94335318e-02,-2.65000000e+00,-3.06867619e-01,2.92256626e-02
    ,-2.65000000e+00,-3.05809726e-01,3.87593382e-02,-2.65000000e+00
    ,-3.04449033e-01,4.82976354e-02,-2.65000000e+00,-3.02783344e-01
    ,5.78283804e-02,-2.65000000e+00,-3.00815326e-01,6.73202402e-02
    ,-2.65000000e+00,-2.98517669e-01,7.68705181e-02,-2.65000000e+00
    ,-2.95981405e-01,8.61213274e-02,-2.65000000e+00,-2.93145345e-01
    ,9.53293329e-02,-2.65000000e+00,-3.21695068e-01,-4.23749178e-04
    ,-2.65000000e+00,-3.21549246e-01,9.69425671e-03,-2.65000000e+00
    ,-3.21086018e-01,1.97905308e-02,-2.65000000e+00,-3.20295862e-01
    ,2.99742760e-02,-2.65000000e+00,-3.19208197e-01,3.99252170e-02
    ,-2.65000000e+00,-3.17800757e-01,4.99056637e-02,-2.65000000e+00
    ,-3.16073925e-01,5.98762886e-02,-2.65000000e+00,-3.14036995e-01
    ,6.97757960e-02,-2.65000000e+00,-3.11663478e-01,7.97105547e-02
    ,-2.65000000e+00,-3.09032535e-01,8.93688349e-02,-2.65000000e+00
    ,-3.06081136e-01,9.90062335e-02,-2.65000000e+00,-3.35132999e-01
    ,-1.00634313e-03,-2.65000000e+00,-3.34998416e-01,9.54991733e-03
    ,-2.65000000e+00,-3.34533424e-01,2.00630976e-02,-2.65000000e+00
    ,-3.33729977e-01,3.06503136e-02,-2.65000000e+00,-3.32615239e-01
    ,4.10151489e-02,-2.65000000e+00,-3.31164382e-01,5.14323980e-02
    ,-2.65000000e+00,-3.29380093e-01,6.18376400e-02,-2.65000000e+00
    ,-3.27276202e-01,7.21486448e-02,-2.65000000e+00,-3.24826442e-01
    ,8.24798278e-02,-2.65000000e+00,-3.22104010e-01,9.25426713e-02
    ,-2.65000000e+00,-3.19041399e-01,1.02604703e-01,-2.65000000e+00
    ,-3.48569540e-01,-1.69720713e-03,-2.65000000e+00,-3.48449725e-01
    ,9.29485424e-03,-2.65000000e+00,-3.47986426e-01,2.02250471e-02
    ,-2.65000000e+00,-3.47172508e-01,3.12226654e-02,-2.65000000e+00
    ,-3.46034177e-01,4.19994479e-02,-2.65000000e+00,-3.44543573e-01
    ,5.28519779e-02,-2.65000000e+00,-3.42705695e-01,6.36899654e-02
    ,-2.65000000e+00,-3.40538241e-01,7.44131136e-02,-2.65000000e+00
    ,-3.38013937e-01,8.51480081e-02,-2.65000000e+00,-3.35203760e-01
    ,9.56140363e-02,-2.65000000e+00,-3.32033799e-01,1.06099772e-01
    ,-2.65000000e+00,-3.62003803e-01,-2.55724076e-03,-2.65000000e+00
    ,-3.61904098e-01,8.87224084e-03,-2.65000000e+00,-3.61447702e-01
    ,2.02200741e-02,-2.65000000e+00,-3.60628467e-01,3.16291175e-02
    ,-2.65000000e+00,-3.59471976e-01,4.28157797e-02,-2.65000000e+00
    ,-3.57946976e-01,5.41041143e-02,-2.65000000e+00,-3.56060534e-01
    ,6.53772828e-02,-2.65000000e+00,-3.53834564e-01,7.65140093e-02
    ,-2.65000000e+00,-3.51240649e-01,8.76544320e-02,-2.65000000e+00
    ,-3.48348416e-01,9.85224527e-02,-2.65000000e+00,-3.45076306e-01
    ,1.09433247e-01,-2.65000000e+00,-3.75435681e-01,-3.50025154e-03
    ,-2.65000000e+00,-3.75358755e-01,8.36703606e-03,-2.65000000e+00
    ,-3.74911917e-01,2.01309906e-02,-2.65000000e+00,-3.74090193e-01
    ,3.19488664e-02,-2.65000000e+00,-3.72918084e-01,4.35465825e-02
    ,-2.65000000e+00,-3.71361005e-01,5.52739223e-02,-2.65000000e+00
    ,-3.69428573e-01,6.69830701e-02,-2.65000000e+00,-3.67146949e-01
    ,7.85322899e-02,-2.65000000e+00,-3.64486618e-01,9.00761221e-02
    ,-2.65000000e+00,-3.61514658e-01,1.01347693e-01,-2.65000000e+00
    ,-3.58142188e-01,1.12687069e-01,-2.65000000e+00,-3.88868458e-01
    ,-4.20205053e-03,-2.65000000e+00,-3.88806656e-01,8.10673180e-03
    ,-2.65000000e+00,-3.88361793e-01,2.02842883e-02,-2.65000000e+00
    ,-3.87530048e-01,3.25084045e-02,-2.65000000e+00,-3.86335067e-01
    ,4.45146128e-02,-2.65000000e+00,-3.84738187e-01,5.66821128e-02
    ,-2.65000000e+00,-3.82751628e-01,6.88296857e-02,-2.65000000e+00
    ,-3.80407253e-01,8.07877244e-02,-2.65000000e+00,-3.77673475e-01
    ,9.27312279e-02,-2.65000000e+00,-3.74615021e-01,1.04402684e-01
    ,-2.65000000e+00,-3.71134386e-01,1.16170574e-01,-2.65000000e+00
    ,-4.02314362e-01,-3.58366431e-03,-2.65000000e+00,-4.02225947e-01
    ,9.16387998e-03,-2.65000000e+00,-4.01741933e-01,2.17510556e-02
    ,-2.65000000e+00,-4.00858320e-01,3.43845293e-02,-2.65000000e+00
    ,-3.99599698e-01,4.67949813e-02,-2.65000000e+00,-3.97921814e-01
    ,5.93962886e-02,-2.65000000e+00,-3.95840331e-01,7.19730582e-02
    ,-2.65000000e+00,-3.93393208e-01,8.43295483e-02,-2.65000000e+00
    ,-3.90544164e-01,9.66692556e-02,-2.65000000e+00,-3.87359615e-01
    ,1.08730022e-01,-2.65000000e+00,-3.83731109e-01,1.20913708e-01
    ,-2.65000000e+00,-4.15768030e-01,-1.10027660e-03,-2.65000000e+00
    ,-4.15593721e-01,1.20881729e-02,-2.65000000e+00,-4.15012123e-01
    ,2.50839189e-02,-2.65000000e+00,-4.14017518e-01,3.81282023e-02
    ,-2.65000000e+00,-4.12638247e-01,5.09307628e-02,-2.65000000e+00
    ,-4.10821357e-01,6.39537190e-02,-2.65000000e+00,-4.08586066e-01
    ,7.69525289e-02,-2.65000000e+00,-4.05978646e-01,8.96972936e-02
    ,-2.65000000e+00,-4.02955711e-01,1.02425390e-01,-2.65000000e+00
    ,-3.99590659e-01,1.14854564e-01,-2.65000000e+00,-3.95758898e-01
    ,1.27432962e-01,-2.65000000e+00,-4.24519193e-01,2.06739680e-03
    ,-2.65000000e+00,-4.24239530e-01,1.55447867e-02,-2.65000000e+00
    ,-4.23545748e-01,2.88065678e-02,-2.65000000e+00,-4.22429160e-01
    ,4.21239121e-02,-2.65000000e+00,-4.20923849e-01,5.51718441e-02
    ,-2.65000000e+00,-4.18969245e-01,6.84513765e-02,-2.65000000e+00
    ,-4.16584697e-01,8.17190886e-02,-2.65000000e+00,-4.13824857e-01
    ,9.47090637e-02,-2.65000000e+00,-4.10638724e-01,1.07687776e-01
    ,-2.65000000e+00,-4.07110800e-01,1.20339584e-01,-2.65000000e+00
    ,-4.03102884e-01,1.33149856e-01,-2.65000000e+00,-4.26606482e-01
    ,2.90146585e-03,-2.65000000e+00,-4.26299162e-01,1.64479026e-02
    ,-2.65000000e+00,-4.25576157e-01,2.97732036e-02,-2.65000000e+00
    ,-4.24427940e-01,4.31559166e-02,-2.65000000e+00,-4.22890190e-01
    ,5.62618557e-02,-2.65000000e+00,-4.20900359e-01,6.96017008e-02
    ,-2.65000000e+00,-4.18477693e-01,8.29332830e-02,-2.65000000e+00
    ,-4.15679053e-01,9.59814245e-02,-2.65000000e+00,-4.12451455e-01
    ,1.09019753e-01,-2.65000000e+00,-4.08882417e-01,1.21723780e-01
    ,-2.65000000e+00,-4.04830273e-01,1.34588111e-01,-2.65000000e+00
    ,-4.27087747e-01,3.09501505e-03,-2.65000000e+00,-4.26774009e-01
    ,1.66573730e-02,-2.65000000e+00,-4.26044227e-01,2.99973199e-02
    ,-2.65000000e+00,-4.24888676e-01,4.33951041e-02,-2.65000000e+00
    ,-4.23343411e-01,5.65144104e-02,-2.65000000e+00,-4.21345421e-01
    ,6.98681535e-02,-2.65000000e+00,-4.18913925e-01,8.32144568e-02
    ,-2.65000000e+00,-4.16106301e-01,9.62760038e-02,-2.65000000e+00
    ,-4.12869102e-01,1.09328071e-01,-2.65000000e+00,-4.09290550e-01
    ,1.22044124e-01,-2.65000000e+00,-4.05228172e-01,1.34920906e-01
    ,-2.65000000e+00,-4.27162736e-01,3.12521537e-03,-2.65000000e+00
    ,-4.26847997e-01,1.66900542e-02,-2.65000000e+00,-4.26117158e-01
    ,3.00322831e-02,-2.65000000e+00,-4.24960464e-01,4.34324157e-02
    ,-2.65000000e+00,-4.23414026e-01,5.65538047e-02,-2.65000000e+00
    ,-4.21414763e-01,6.99097129e-02,-2.65000000e+00,-4.18981890e-01
    ,8.32583099e-02,-2.65000000e+00,-4.16172864e-01,9.63219455e-02
    ,-2.65000000e+00,-4.12934168e-01,1.09376153e-01,-2.65000000e+00
    ,-4.09354133e-01,1.22094080e-01,-2.65000000e+00,-4.05290159e-01
    ,1.34972801e-01};

  for (int i = 0; i < 231; i++) {
    vtxCoordB[2*i] += coeffrand * random01();
  }


  PDM_g_num_t vtxLNToGNB[231];
  for (int i = 0; i < 231; ++i) {
    vtxLNToGNB[i] = i+1;
  }

  int nFace[2][1];

  nFace[0][0] = nFaceA_merge;
  nFace[1][0] = nFaceB;

  PDM_g_num_t nGFace[2] = {nFaceA_merge, nFaceB};
  PDM_g_num_t nGVtx[2] = {nVtxA_merge, nVtxB};

  double projectCoeff = 1.;

  //
  // meshes intersection
  //

  int pdm_id = PDM_ol_create (n_part,
                              nGFace[0],
                              nGVtx[0],
                              n_part,
                              nGFace[1],
                              nGVtx[1],
                              projectCoeff,
                              PDM_MPI_COMM_WORLD);

  PDM_ol_parameter_set (pdm_id,
                        PDM_OL_CAR_LENGTH_TOL,
                        1e-3);

  PDM_ol_parameter_set (pdm_id,
                        PDM_OL_EXTENTS_TOL,
                        1e-3);

  PDM_ol_input_mesh_set (pdm_id,
                         PDM_OL_MESH_A,
                         0,
                         nFaceA_merge,
                         faceVtxIdxA_merge,
                         faceVtxA_merge,
                         faceLNToGNA_merge,
                         nVtxA_merge,
                         vtxCoordA_merge,
                         vtxLNToGNA_merge);


  PDM_ol_input_mesh_set (pdm_id,
                         PDM_OL_MESH_B,
                         0,
                         nFaceB,
                         faceVtxIdxB,
                         faceVtxB,
                         faceLNToGNB,
                         nVtxB,
                         vtxCoordB,
                         vtxLNToGNB);

  PDM_ol_compute (pdm_id);


  PDM_ol_dump_times (pdm_id);

  //
  // Field exchange
  //

  double *sFieldA = malloc(sizeof(double) * nFaceA_merge);
  double *centerA = malloc(sizeof(double) * 3 * nFaceA_merge);
  for (int i = 0; i < 3 * nFaceA_merge; i++) {
    centerA[i] = 0.;
  }

  double y0 = -3.21215e-1;
  double z0 = 3.89064e-2;
  double r = 0.02;
  double A = 100.;

  for (int i = 0; i < nFaceA; i++) {
    for (int j = faceVtxIdxA_merge[i]; j < faceVtxIdxA_merge[i+1]; j++) {
      int    k = faceVtxA_merge[j] - 1;
      double x = vtxCoordA_merge[3*k  ];
      double y = vtxCoordA_merge[3*k+1];
      double z = vtxCoordA_merge[3*k+2];
      centerA[3*i  ] += x;
      centerA[3*i+1] += y;
      centerA[3*i+2] += z;
    }

    int nVtxElt = faceVtxIdxA_merge[i+1] - faceVtxIdxA_merge[i];
    centerA[3*i  ] /= nVtxElt;
    centerA[3*i+1] /= nVtxElt;
    centerA[3*i+2] /= nVtxElt;

    double _y = centerA[3*i+1]  - y0;
    double _z = centerA[3*i+2]- z0;

    sFieldA[i]          = A * exp((-(_y*_y)-(_z*_z))/(2*r*r));
    //sFieldA[i] = 1.5;
    sFieldA[i+nFaceA]   = sFieldA[i];
    sFieldA[i+2*nFaceA] = sFieldA[i];
  }

  free (centerA);

  int           nOlFaceA;
  int           nOlLinkedFaceA;
  int           nOlVtxA;
  int           sOlFaceIniVtxA;
  int           sOlface_vtxA;
  int           sInitToOlFaceA;

  PDM_ol_part_mesh_dim_get (pdm_id,
                            PDM_OL_MESH_A,
                            0,
                            &nOlFaceA,
                            &nOlLinkedFaceA,
                            &nOlVtxA,
                            &sOlFaceIniVtxA,
                            &sOlface_vtxA,
                            &sInitToOlFaceA);

  int            *olFaceIniVtxIdxA;
  int            *olFaceIniVtxA;
  int            *olface_vtx_idxA;
  int            *olface_vtxA;
  int            *olLinkedface_procIdxA;
  int            *olLinkedFaceA;
  PDM_g_num_t     *olface_ln_to_gnA;
  double         *olCoordsA;
  PDM_g_num_t     *olvtx_ln_to_gnA;
  int            *initToOlFaceIdxA;
  int            *initToOlFaceA;

  PDM_ol_mesh_entities_get (pdm_id,
                            PDM_OL_MESH_A,
                            0,
                            &olFaceIniVtxIdxA,
                            &olFaceIniVtxA,
                            &olface_vtx_idxA,
                            &olface_vtxA,
                            &olLinkedface_procIdxA,
                            &olLinkedFaceA,
                            &olface_ln_to_gnA,
                            &olCoordsA,
                            &olvtx_ln_to_gnA,
                            &initToOlFaceIdxA,
                            &initToOlFaceA);


  int           nOlFaceB;
  int           nOlLinkedFaceB;
  int           nOlVtxB;
  int           sOlFaceIniVtxB;
  int           sOlface_vtxB;
  int           sInitToOlFaceB;

  PDM_ol_part_mesh_dim_get (pdm_id,
                            PDM_OL_MESH_B,
                            0,
                            &nOlFaceB,
                            &nOlLinkedFaceB,
                            &nOlVtxB,
                            &sOlFaceIniVtxB,
                            &sOlface_vtxB,
                            &sInitToOlFaceB);

  int            *olFaceIniVtxIdxB;
  int            *olFaceIniVtxB;
  int            *olface_vtx_idxB;
  int            *olface_vtxB;
  int            *olLinkedface_procIdxB;
  int            *olLinkedFaceB;
  PDM_g_num_t     *olface_ln_to_gnB;
  double         *olCoordsB;
  PDM_g_num_t     *olvtx_ln_to_gnB;
  int            *initToOlFaceIdxB;
  int            *initToOlFaceB;

  PDM_ol_mesh_entities_get (pdm_id,
                            PDM_OL_MESH_B,
                            0,
                            &olFaceIniVtxIdxB,
                            &olFaceIniVtxB,
                            &olface_vtx_idxB,
                            &olface_vtxB,
                            &olLinkedface_procIdxB,
                            &olLinkedFaceB,
                            &olface_ln_to_gnB,
                            &olCoordsB,
                            &olvtx_ln_to_gnB,
                            &initToOlFaceIdxB,
                            &initToOlFaceB);

  double *sFieldOlA = malloc(sizeof(double) * nOlFaceA);
  double *rFieldOlB = malloc(sizeof(double) * nOlFaceB);

  for (int i = 0; i < nOlFaceB; i++) {
    rFieldOlB[i] = 0.;
  }

  for (int i = 0; i < nFaceA_merge; i++) {
    for (int j = initToOlFaceIdxA[i]; j < initToOlFaceIdxA[i+1]; j++) {
      sFieldOlA[initToOlFaceA[j]-1] = sFieldA[i];
    }
  }

  for (int i = 0; i < nOlLinkedFaceB; i++) {
    rFieldOlB[olLinkedFaceB[4*i]-1] = sFieldOlA[olLinkedFaceB[4*i+3]-1];
  }

  // Calcul des surfaces a faire

  double *surfB = malloc(sizeof(double) * nFaceB);
  double *surfOlB = malloc(sizeof(double) * nOlFaceB);

  double   *ol_surface_vector = malloc(sizeof(double) * 3 * nOlFaceB);
  double   *ol_center = malloc(sizeof(double) * 3 * nOlFaceB);
  double   *ol_characteristicLength = malloc(sizeof(double) * nOlFaceB);
  int      *ol_isDegenerated = malloc(sizeof(int) * nOlFaceB);

  PDM_geom_elem_polygon_properties(nOlFaceB,
                                   olface_vtx_idxB,
                                   olface_vtxB,
                                   olCoordsB,
                                   ol_surface_vector,
                                   ol_center,
                                   ol_characteristicLength,
                                   ol_isDegenerated);

  for (int i = 0; i < nOlFaceB; i++) {
    surfOlB[i] = PDM_MODULE(ol_surface_vector + 3*i);
  }

  for (int i = 0; i < nFaceB; i++) {
    surfB[i] = 0.;
    for (int j = initToOlFaceIdxB[i]; j < initToOlFaceIdxB[i+1]; j++) {
      surfB[i] += surfOlB[initToOlFaceB[j]-1];
    }
  }

  //
  // Definir le champ sur B

  double *rFieldB = malloc(sizeof(double) * nFaceB);

  for (int i = 0; i < nFaceB; i++) {
    rFieldB[i] = 0.;
    for (int j = initToOlFaceIdxB[i]; j < initToOlFaceIdxB[i+1]; j++) {
      rFieldB[i] += rFieldOlB[initToOlFaceB[j]-1] *
        surfOlB[initToOlFaceB[j]-1];
    }
    rFieldB[i] /= surfB[i];
  }

  //
  // Export meshes to ensight
  //

  int ens_meshA = PDM_writer_create ("Ensight",
                                     PDM_WRITER_FMT_ASCII,
                                     PDM_WRITER_TOPO_CONSTANTE,
                                     PDM_WRITER_OFF,
                                     "test_2d_unit_ens",
                                     "meshA",
                                     PDM_MPI_COMM_WORLD,
                                     PDM_IO_ACCES_MPI_SIMPLE,
                                     1.,
                                     NULL);

  int id_var_fieldA = PDM_writer_var_create (ens_meshA,
                                             PDM_WRITER_OFF,
                                             PDM_WRITER_VAR_SCALAIRE,
                                             PDM_WRITER_VAR_ELEMENTS,
                                             "sfieldA");
  int ens_geoA_merge = PDM_writer_geom_create (ens_meshA,
                                         "meshA_merge",
                                         PDM_WRITER_OFF,
                                         PDM_WRITER_OFF,
                                         1);
  PDM_writer_step_beg (ens_meshA, 0.);

  PDM_writer_geom_coord_set (ens_meshA,
                             ens_geoA_merge,
                             0,
                             nVtxA_merge,
                             vtxCoordA_merge,
                             vtxLNToGNA_merge);

  int *faceVtxNA_merge =  malloc(sizeof(int) * nFaceA_merge);
  for (int j = 0; j < nFaceA_merge; j++) {
    faceVtxNA_merge[j]  = faceVtxIdxA_merge[j+1] - faceVtxIdxA_merge[j];
  }

  PDM_writer_geom_faces_facesom_add (ens_meshA,
                                     ens_geoA_merge,
                                     0,
                                     nFaceA_merge,
                                     faceVtxIdxA_merge,
                                     faceVtxNA_merge,
                                     faceVtxA_merge,
                                     faceLNToGNA_merge);

  PDM_writer_geom_write(ens_meshA,
                        ens_geoA_merge);

  //

  int ens_geoA = PDM_writer_geom_create (ens_meshA,
                                         "meshA",
                                         PDM_WRITER_OFF,
                                         PDM_WRITER_OFF,
                                         1);
  PDM_writer_geom_coord_set (ens_meshA,
                             ens_geoA,
                             0,
                             nVtxA,
                             vtxCoordA,
                             vtxLNToGNA);

  int *faceVtxNA =  malloc(sizeof(int) * nFaceA);
  for (int j = 0; j < nFaceA; j++) {
    faceVtxNA[j]  = faceVtxIdxA[j+1] - faceVtxIdxA[j];
  }

  PDM_writer_geom_faces_facesom_add (ens_meshA,
                                     ens_geoA,
                                     0,
                                     nFaceA,
                                     faceVtxIdxA,
                                     faceVtxNA,
                                     faceVtxA,
                                     faceLNToGNA);

  PDM_writer_geom_write(ens_meshA,
                        ens_geoA);
  //

  PDM_writer_var_set (ens_meshA,
                      id_var_fieldA,
                      ens_geoA,
                      0,
                      sFieldA);

  PDM_writer_var_set (ens_meshA,
                      id_var_fieldA,
                      ens_geoA_merge,
                      0,
                      sFieldA);

  PDM_writer_var_write (ens_meshA,
                        id_var_fieldA);

  PDM_writer_var_free (ens_meshA,
                       id_var_fieldA);


  PDM_writer_step_end (ens_meshA);
  PDM_writer_geom_data_free (ens_meshA,
                             ens_geoA);

  PDM_writer_geom_free (ens_meshA,
                        ens_geoA);

  PDM_writer_geom_data_free (ens_meshA,
                             ens_geoA_merge);

  PDM_writer_geom_free (ens_meshA,
                        ens_geoA_merge);

  PDM_writer_free (ens_meshA);
  free (faceVtxNA);
  free (faceVtxNA_merge);


  int ens_meshB = PDM_writer_create ("Ensight",
                                     PDM_WRITER_FMT_ASCII,
                                     PDM_WRITER_TOPO_CONSTANTE,
                                     PDM_WRITER_OFF,
                                     "test_2d_unit_ens",
                                     "meshB",
                                     PDM_MPI_COMM_WORLD,
                                     PDM_IO_ACCES_MPI_SIMPLE,
                                     1.,
                                     NULL);

  int id_var_fieldB = PDM_writer_var_create (ens_meshB,
                                             PDM_WRITER_OFF,
                                             PDM_WRITER_VAR_SCALAIRE,
                                             PDM_WRITER_VAR_ELEMENTS,
                                             "rfieldB");

  int ens_geoB = PDM_writer_geom_create (ens_meshB,
                                         "meshB",
                                         PDM_WRITER_OFF,
                                         PDM_WRITER_OFF,
                                         1);
  PDM_writer_step_beg (ens_meshB, 0.);

  PDM_writer_geom_coord_set (ens_meshB,
                             ens_geoB,
                             0,
                             nVtxB,
                             vtxCoordB,
                             vtxLNToGNB);

  int *faceVtxNB =  malloc(sizeof(int) * nFaceB);
  for (int j = 0; j < nFaceB; j++) {
    faceVtxNB[j]  = faceVtxIdxB[j+1] - faceVtxIdxB[j];
  }


  PDM_writer_geom_faces_facesom_add (ens_meshB,
                                     ens_geoB,
                                     0,
                                     nFaceB,
                                     faceVtxIdxB,
                                     faceVtxNB,
                                     faceVtxB,
                                     faceLNToGNB);

  PDM_writer_geom_write(ens_meshB,
                        ens_geoB);


  PDM_writer_var_set (ens_meshB,
                      id_var_fieldB,
                      ens_geoB,
                      0,
                      rFieldB);

  PDM_writer_var_write (ens_meshB,
                        id_var_fieldB);

  PDM_writer_var_free (ens_meshB,
                       id_var_fieldB);

  PDM_writer_step_end (ens_meshB);
  PDM_writer_geom_data_free (ens_meshB,
                             ens_geoB);

  PDM_writer_geom_free (ens_meshB,
                        ens_geoB);

  PDM_writer_free (ens_meshB);
  free (faceVtxNB);


  _export_ol_mesh (PDM_MPI_COMM_WORLD,
                   pdm_id,
                   nFace,
                   &sFieldOlA,
                   &rFieldOlB,
                   1);

  //
  // Free
  //

  free (sFieldA);
  free (rFieldB);
  free (sFieldOlA);
  free (rFieldOlB);
  free (surfB);
  free (surfOlB);
  free (ol_surface_vector);
  free (ol_center);
  free (ol_characteristicLength);
  free (ol_isDegenerated);

  free (olFaceIniVtxIdxA);
  free (olFaceIniVtxA);
  free (olface_vtx_idxA);
  free (olface_vtxA);
  free (olLinkedface_procIdxA);
  free (olLinkedFaceA);
  free (olface_ln_to_gnA);
  free (olCoordsA);
  free (olvtx_ln_to_gnA);
  free (initToOlFaceIdxA);
  free (initToOlFaceA);

  free (olFaceIniVtxIdxB);
  free (olFaceIniVtxB);
  free (olface_vtx_idxB);
  free (olface_vtxB);
  free (olLinkedface_procIdxB);
  free (olLinkedFaceB);
  free (olface_ln_to_gnB);
  free (olCoordsB);
  free (olvtx_ln_to_gnB);
  free (initToOlFaceIdxB);
  free (initToOlFaceB);

  free (faceVtxIdxA_merge);
  free (faceVtxA_merge);
  free (vtxCoordA_merge);
  free (vtxLNToGNA_merge);
  free (faceLNToGNA_merge);

  PDM_ol_del (pdm_id);

  PDM_MPI_Finalize ();

  return 0;

}
